import os
from typing import Optional
import allure

from mag_tools.utils.common.string_utils import StringUtils

from mag_tools.model.log_type import LogType

from mag_tools.log.logger import Logger
from mag_test.model.test_component_type import TestComponentType

from mag_test.model.test_result import TestResult


class BaseTest:
    def __init__(self, home_dir, name:str, index:Optional[int]=None, test_component_type:Optional[TestComponentType] = None, description:Optional[str]=None):
        self._name = name
        self._index = index
        self._test_component_type = test_component_type
        self._description = description
        self._test_result = TestResult.SKIP
        self._err_message = None
        self._home_dir = home_dir

    def _report(self):
        Logger.info(LogType.FRAME, f"执行{self._test_component_type.desc}({self._name})完毕")

        message = f"{self._test_result.desc}: {self._err_message}" if self._test_result == TestResult.FAIL else self._test_result.desc
        if self._test_component_type == TestComponentType.MODULE:
            index_ch = StringUtils.to_chinese_number(self._index) + "、" if self._index else ""
            allure.dynamic.feature(f"{index_ch}{self._name}")  # 所属功能模块
        elif self._test_component_type == TestComponentType.CASE:
            allure.dynamic.title(self._name)  # 测试用例名（标题）
            if self._description:
                allure.dynamic.story(self._description)  # 测试用例描述
                allure.dynamic.story(message)  # 用例测试结果
        elif self._test_component_type == TestComponentType.FUNCTION:
            title = self._description if self._description else (self._name if self._name else '')
            allure.step(f"{self._index} {title}")  # 描述测试功能
            allure.step(f"\t{message}")    # 功能测试结果
        elif self._test_component_type == TestComponentType.STEP:
            title = self._description if self._description else (self._name if self._name else '')
            allure.step(f"  {self._index} {title}")  # 描述测试步骤
            allure.step(f"  \t{message}")  # 测试步骤测试结果

    def start(self, driver):
        raise NotImplementedError("This method should be overridden in subclasses")

    def skip(self):
        Logger.info(f"{self._test_component_type.desc}-{self._name}未测试")

    def fail(self, message:Optional[str]=''):
        self._test_result = TestResult.FAIL
        self._err_message = message
        Logger.error(f"{self._test_component_type.desc}-{self._name}失败：{message}")

    def success(self):
        self._test_result = TestResult.SUCCESS
        Logger.info(f"{self._test_component_type.desc}-{self._name}成功")

    def is_success(self):
        return self._test_result == TestResult.SUCCESS

    def is_fail(self):
        return self._test_result == TestResult.FAIL

    @property
    def script_dir(self):
        return os.path.join(self._home_dir, 'script')

    def get_attachment(self, attachment_name):
        attachment_dir = os.path.join(self._home_dir, 'attachment')
        return os.path.join(attachment_dir, attachment_name)


import os.path

from mag_tools.utils.file.file_utils import FileUtils
from mag_tools.utils.file.json_file_utils import JsonFileUtils

from mag_test.bean.report_info import ReportInfo


class ReportStyle:
    @staticmethod
    def update(report_info: ReportInfo):
        ReportStyle.__update_window_title(report_info.root_dir, report_info.report_dir, report_info.corporation_name)
        ReportStyle.__update_overview_title(report_info.report_dir, report_info.name)

    # 修改报告的窗口标题
    @staticmethod
    def __update_window_title(root_dir: str, report_dir:str, corporation_name):
        # 读取html文件
        with open(os.path.join(report_dir, 'index.html'), 'r+', encoding='utf-8') as f:
            # 先保存html的内容
            lines = f.readlines()
            # 定义下标变量，方便后边的数据更替
            i = -1
            for line in lines:
                i += 1
                # 寻找html页面的title标签
                if '<title>' in line:
                    # 定义原始标题变量
                    old_title = ''
                    # html里的title标签之前有4个空格，所以从下标为11处截取
                    for s in line[11:]:
                        # 若字符为<则退出循环
                        if s == '<':
                            break
                        # 原始标题-拨云见日
                        old_title += s
                    # 正式替换标题
                    lines[i] = line.replace(old_title, corporation_name)
                    # 修改标题后的html内容
                    new_lines = lines
            # 指针放在文件开始的地方
            f.seek(0)
            # 清空原来的html文件内容
            f.truncate()
            # 将修改标题后的内容重新写入html文件
            for new_line in new_lines:
                f.write(new_line)

        # 替换favicon.ico文件
        FileUtils.copy_and_overwrite(os.path.join(root_dir, 'data', 'favicon.ico'), os.path.join(report_dir, 'favicon.ico'))

    # 修改报告的Overview标题
    @staticmethod
    def __update_overview_title(report_dir, report_name):
        JsonFileUtils.update_params(os.path.join(report_dir, 'widgets', 'summary.json'), {'reportName': report_name})


from sys import exception
from typing import Optional

import pytest
from mag_tools.exception.app_exception import AppException
from mag_tools.log.logger import Logger
from mag_tools.model.convert_type import ConvertType
from mag_tools.model.data_type import DataType
from mag_tools.model.common.message_type import MessageType
from mag_tools.utils.common.string_format import StringFormat
from mag_tools.utils.file.file_utils import FileUtils
from mag_tools.utils.test.table_utils import TableUtils
from mag_tools.utils.test.event_utils import EventUtils
from mag_tools.utils.file.json_file_utils import JsonFileUtils
from mag_tools.utils.common.string_utils import StringUtils

from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver import ActionChains

from mag_test.model.control_type import ControlType
from mag_test.model.test_component_type import TestComponentType
from mag_test.bean.base_test import BaseTest
from mag_test.bean.element_info import ElementInfo
from mag_test.core.app_driver import AppDriver
from mag_test.finder.element_finder import ElementFinder


class Step(BaseTest):
    def __init__(self, home_dir:str, name: Optional[str], control_name:Optional[str], control_type:Optional[ControlType],
                 automation_id:Optional[str], value:Optional[str], function_index:Optional[int]=None, step_index:Optional[int]=None,
                 parent_name:Optional[str]=None, parent_type:Optional[ControlType]=None, parent_id:Optional[str]=None,
                 pop_window:Optional[str]=None):
        super().__init__(home_dir, name, step_index, TestComponentType.STEP, None)

        self.__function_index = function_index
        self.__element_info = ElementInfo(control_name, control_type, automation_id, None,
                                          parent_name, parent_type, parent_id, None, StringFormat.format(value), pop_window)

    @pytest.mark.benchmark
    def start(self, driver:AppDriver):
        """
        启动测试步骤
        :param driver: AppDriver
        """
        try:
            Logger.debug(f'测试步骤[{self._name}]-{self._index}：\n\t{self.__element_info}')

            if self.__element_info.is_virtual_control():
                if self.__element_info.control_type == ControlType.FILE:
                    action = self.__element_info.name
                    path = self.__element_info.value
                    if action and action.upper() == 'CLEAR_DIR':
                        FileUtils.clear_dir(path)
            else:
                # 查找控件并处理事件
                element = ElementFinder.find(driver, self.__element_info)
                self.__process_event(driver, element)

                # 检查消息提示框
                alert_result = driver.check_alert()
                if alert_result[0] in {MessageType.ERROR}:
                    raise AppException(alert_result[1])

                # 如果指定了弹出窗口，则切换
                if self.__element_info.pop_window:
                    driver = ElementFinder.switch_to_window_by_title(driver, self.__element_info.pop_window)

            self.success()
        except (AppException, Exception) as e:
            Logger.error(f"测试步骤[{self._name}-{self._index}]失败: {self.__element_info.name()}({self.__element_info.control_type})\n{str(e)}")
            self.fail(str(e))

        super()._report()
        return driver

    def __process_event(self, driver:AppDriver, element:WebElement):
        """
        启动测试步骤
        :param driver: AppDriver
        """
        if element is None:
            raise exception(f'未找到指定的控件 {self.__element_info}')

        return_type = ControlType.get_by_element(element)
        Logger.debug(f'找到的控件返回类型为：{self.__element_info.control_type} ({return_type})')

        if return_type in {ControlType.BUTTON, ControlType.SPLIT_BUTTON, ControlType.MENU_ITEM,
                            ControlType.TOOL, ControlType.RADIO}:  # 单击控件
            element.click()
        elif return_type in {ControlType.EDIT, ControlType.DOC}:  # 可编辑控件
            actions = ActionChains(driver)
            actions.move_to_element(element).click().perform()
            element.clear()
            element.send_keys(self.__element_info.value)
        elif return_type in {ControlType.LABEL}:  # 双击控件
            actions = ActionChains(driver)
            actions.move_to_element(element).double_click().perform()
        elif return_type in {ControlType.TREE_ITEM,
                              ControlType.TAB_ITEM,
                              ControlType.LIST_ITEM}:  # 复杂交互的点击
            offset = self.__element_info.get_offset(element.size['width'], element.size['height']) if self.__element_info.value else None
            EventUtils.click(driver, element, offset)
        elif return_type == ControlType.TABLE:
            if self.__element_info.value:
                convert_type = ConvertType.of_code(self.__element_info.child_name)
                attachment_file = self.get_attachment(self.__element_info.value)
                value_array = JsonFileUtils.load_json(attachment_file)
                TableUtils.set_table(element, value_array, convert_type)
        elif return_type in {ControlType.CHECKBOX}:  # 复选按钮选择
            value = StringUtils.to_value(self.__element_info.value, DataType.BOOLEAN)
            if value and not element.is_selected():
                element.click()
            elif not value and element.is_selected():
                element.click()
        elif return_type == ControlType.PANE:
            if self.__element_info.control_type == ControlType.DATETIME:
                element.send_keys(self.__element_info.value)
            else:
                element.clear()
        elif return_type == ControlType.WINDOW:
            if self.__element_info.value == 'close':
                driver.close()
                driver.quit_app()
            elif self.__element_info.value == 'max':
                driver.maximize_window()
            elif self.__element_info.value == 'min':
                driver.minimize_window()
        elif return_type == ControlType.COMBO_BOX:
            element.click()
            element.clear()
            element.send_keys(self.__element_info.value)
        else:
            Logger.info(f"Unsupported type or action: {self.__element_info.control_type} : {return_type}")